@@ -716,8 +716,14 @@ StatusTuple CodegenLLVM::emit_atomic_add(MethodCallExprNode *n) {
   Value *lhs = B.CreateBitCast(pop_expr(), Type::getInt64PtrTy(ctx()));
   TRY2(n->args_[1]->accept(this));
   Value *rhs = B.CreateSExt(pop_expr(), B.getInt64Ty());
+#if LLVM_MAJOR_VERSION >= 13
+  AtomicRMWInst *atomic_inst = B.CreateAtomicRMW(
+      AtomicRMWInst::Add, lhs, rhs, Align(8),
+      AtomicOrdering::SequentiallyConsistent);
+#else
   AtomicRMWInst *atomic_inst = B.CreateAtomicRMW(
       AtomicRMWInst::Add, lhs, rhs, AtomicOrdering::SequentiallyConsistent);
+#endif
   atomic_inst->setVolatile(false);
   return StatusTuple::OK();
 }
